// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#endif
#include "IBK_constants.h"

namespace IBK {

const char * const IBK_LIB_VERSION       = "3.8.4";

const double POS_INF    = 1e30;
const double NEAR_ZERO  = 1e-10;

// numerical constants
const double PI        	= 3.1415926535897932384626433832795;
const double BOLTZMANN 	= 5.67e-08;
const double FARADAY    = 96485.3415; // in [sA/mol]

const double R_IDEAL_GAS 		= 8.314472;     // ideal gas constant in [J/molK]
const double R_VAPOR			= 462;          // gas constant for water vapour in [J/kgK]
const double R_VAPOR_1			= 1.0/R_VAPOR;  // inverse gas constant for water vapour
const double R_AIR				= 287.1;       	// gas constant for dry air in [J/kgK]
const double R_AIR_1			= 1.0/R_AIR; 	// inverse gas constant for dry air
const double RHO_W				= 1000;         // density of water in [kg/m3]
const double RHO_W_1			= 1.0/RHO_W;    // inverse of density of water
const double T_DEFAULT			= 293.15;       // default temperature for isothermal calculations in [K].
const double T_REF				= 273.15;       // reference temperature for calculation of internal energy in [K].
const double C_WATER 			= 4.18e3;       // specific heat capacity of the liquid water in J/(kg K)
const double C_VAPOR			= 2.05e3;       // specific heat capacity of the water vapour in J/(kg K)
const double C_AIR              = 1.006e3;      // specific heat capacity of air in J/(kg K)
const double RHO_AIR            = 1.20;         // density of air in kg/m3
const double H_EVAP				= 2.445e6;      // specific heat of evaporation of the water vapour in J/kg
const double KELVIN_FACTOR		= 1.0/(1000*462*T_DEFAULT);
const double GASPRESS_REF       = 101325;       // reference pressure in Pa
const double GRAVITY 		    = 9.807;        // gravitational constant in [m/s2]
const double MIN_RH 		    = 1e-10;        // minimum relative humidity [-]
const double MIN_PC_T 		    = -23.02585093*RHO_W*R_VAPOR;   // pc/T at MIN_RH
const double T_REF_23           = 296.15;       // reference temperature for transformation of material functions
const double DV_AIR             = 2.662e-5;     // water vapor diffusivity in air at reference temperature

} // namespace IBK

