// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_constants_h
#define IBK_constants_h

namespace IBK {

extern const char * const IBK_LIB_VERSION;

extern const double POS_INF;    ///< Positive infinity limit.
extern const double NEAR_ZERO;  ///< Used for equality check of double numbers.

extern const double PI;         ///< The constant PI.
extern const double BOLTZMANN;  ///< The BOLTZMANN constant (= 5.67e-8 W/m2K4).
extern const double FARADAY;    ///< Faraday's constant (= 96485.3415 sA/mol).

extern const double R_IDEAL_GAS;    // = 8.31;          // universal gas constant in [J/molK]
extern const double R_VAPOR;        // = 461.89;        // gas constant for water vapour in [J/kgK]
extern const double R_VAPOR_1;      // = 1.0/R_VAPOR;  // inverse gas constant for water vapour
extern const double R_AIR;          // = 287.1;         // gas constant for dry air in [J/kgK]
extern const double R_AIR_1;        // = 1.0/R_AIR;     // gas constant for dry air in [J/kgK]
extern const double RHO_W;          // = 1000;          // density of water in [kg/m3]
extern const double RHO_W_1;        // = 1.0/RHO_W;     // inverse of density of water
extern const double T_DEFAULT;      // = 293.15;        // default temperature for isothermal calculations in [K].
extern const double T_REF;          // = 273.15;        // reference temperature for calculation of internal energy.
extern const double C_WATER;        // = 4.18e3;        // specific heat capacity of the liquid water in [J/kgK]
extern const double C_VAPOR;        // = 2.05e3;        // specific heat capacity of the water vapour in [J/kgK]
extern const double C_AIR;          // = 1.006e3;       // specific heat capacity of air in [J/kgK]
extern const double RHO_AIR;        // = 1.20;          // density of air in [kg/m3]
extern const double H_EVAP;         // = 2.445e6;       // specific heat of evaporation of the water vapour in [J/kg]
extern const double KELVIN_FACTOR;  // = 1.0/(1000*462*T_DEFAULT);
extern const double GASPRESS_REF;   // = 101325;        // reference pressure in [Pa]
extern const double GRAVITY;		// = 9.807;         // gravitational constant in [m/s2]
extern const double MIN_RH;		    // = 1e-10;         // minimum relative humidity [-]
extern const double MIN_PC_T;		// = f_log(MIN_RH)*RHO_W*R_VAPOR  // pc/T at MIN_RH
extern const double T_REF_23;       // reference temperature for transformation of material functions
extern const double DV_AIR;         // water vapor diffusivity in air at reference temperature

} // namespace IBK

#endif // IBK_constants_h

