// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <iostream>
  #include <sstream>
  #include <fstream>
  #include <stdexcept>
#endif
#include "IBK_debug.h"
#include "IBK_messages.h"
#include "IBK_exception.h"
using namespace std;

namespace IBK {

/// Assertion (check) function 
void _check(char * cond, char * file, int line) {
	string fname(file);
    stringstream errstrm;
	errstrm << "DEBUG MODE - RUNTIME CHECK FAILED\n"
    	    << "CHECK : " << cond << "\n"
    	    << "FILE  : " << ((fname.size() < 69) ?
                             fname :
	                         ("..." + fname.substr(fname.size()-66, 66))) << "\n"
    	    << "LINE  : " << line << endl;
    if (!_DBCL().m_trace.empty()) {
    	errstrm << "\nCALL TRACE" << endl;
        ofstream trace("call_trace.debug");
        list<string>::reverse_iterator it = _DBCL().m_trace.rbegin();
        while (it != _DBCL().m_trace.rend()) {
            errstrm << "  " << ((it->size() < 75) ? *it : "..."+it->substr(it->size()-72, 72)) << endl;
            trace << *it << endl;
            ++it;
        }
    }
	// TODO : THIS NEEDS FIXING
	MSG(errstrm.str(),MSG_ERROR);
    throw IBK_exception("Assertion failure!", "[IBK::_check]");
//	system("pause");
//    exit(EXIT_FAILURE);
}

_DebugContextList& _DBCL() {
	static _DebugContextList DBCL;
	return DBCL;
}

DebugContext::DebugContext(const char * file, const char * func) {
	_DBCL().m_trace.push_back(string(file) + " | " + func);
}

/// Destructor, removes the last debug trace from the trace list
DebugContext::~DebugContext() {
	if (!_DBCL().m_trace.empty()) {
    	list<string>::iterator it = _DBCL().m_trace.end();
    	_DBCL().m_trace.erase(--it);
    }
}

} // namespace IBK

