// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
//
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_debug_h
#define IBK_debug_h

#include <list>
#include <string>

namespace IBK {
    /// Assertion (check) function
    void _check(char * cond, char * file, int line);

	class _DebugContextList {
	  public:
	  	std::list<std::string> m_trace;
      private:
      	_DebugContextList() {}
      	_DebugContextList(const _DebugContextList&);

      friend _DebugContextList& _DBCL();
	};

	_DebugContextList& _DBCL();

	/// DebugContext class automatically generated by IBK_DEBUG_CONTEXT macro.
	class DebugContext {
	  public:
	  	/// Constructor, adds a debug trace to the trace list
	  	DebugContext(const char * file, const char * func);
	  	/// Destructor, removes the last debug trace from the trace list
	  	~DebugContext();
	};
}
// ----------------------------------------------------------------------------

#ifdef IBK_ENABLE_DEBUG_MACROS

/// Debugging macro similar to standard C assert() but profides additional
/// call trace information.
#define IBK_check(p)   			((p) ? (void)0 : IBK::_check(#p, __FILE__, __LINE__))
/// Macro defining a debug context (call trace information) in the current function.
#ifdef __BORLANDC__
#define IBK_DEBUG_CONTEXT		IBK::DebugContext _dbct_element(__FILE__, __FUNC__);
#else
#define IBK_DEBUG_CONTEXT		IBK::DebugContext _dbct_element(__FILE__, __FUNCTION__);
#endif // __BORLANDC__

#else // IBK_ENABLE_DEBUG_MACROS

/// Debugging macro similar to standard C assert() but profides additional
/// call trace information.
#define IBK_check(p)   ((void)0)
/// Macro defining a debug context (call trace information) in the current function.
#define IBK_DEBUG_CONTEXT ((void)0)

#endif // IBK_ENABLE_DEBUG_MACROS

/// @file IBK_debug.h
/// @brief Contains the debugging macro definitions.

/// @example debug_example.cpp
/// This small example demonstrates the use of the IBK debugging macros system.

#endif	// IBK_debug_h
