// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_fileutils_h
#define IBK_fileutils_h

#include <string>
#include <iosfwd>
#include <stdexcept>
#include "IBK_unitlistmanager.h"
#include "IBK_keywordlistmanager.h"
#include "IBK_format.h"

namespace IBK {

enum dir_exists_t {
	DIR_EXISTS,
	IS_FILE,
	NO_DIR
};

/// @brief Tests if a directory exists.
/// @param dirname The full path name in posix format (no backslashes).
/// @return Returns true if directory exists and can be accessed or returns
///         false if directory does not exist, cannot be accessed or dirname points
///			to a file.
dir_exists_t directory_exists(const std::string& dirname);

/// @brief Creates directory.
/// @param dirname The full path name in posix format (no backslashes).
/// @return Returns true if directory was successfully created.
bool create_directory(const std::string& dirname);

/// @brief Moves a file.
bool move_file(const std::string& source, const std::string& target);

/// @brief Removes .. from relative path names.
void simplify_path(std::string& path);

/// @brief Replaces spaces with Escape sequences.
std::string replace_spaces(const std::string& path);

/// @brief Replaces directory placeholders with absolute paths
std::string replaceDir(const std::string& orig_line,
    const std::string& placeholder, const std::string& replace_str);
}  // namespace IBK


/// @file IBK_fileutils.h
/// @brief Contains helper functions for file access on windows/posix systems.

#endif  // IBK_fileutils_h
