// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2006 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <stdexcept>
  #include <iostream>
  #include <iomanip>
#endif
#include "IBK_flag.h"
#include "IBK_stringutils.h"
#include "IBK_format.h"
#include "IBK_message_strings.h"
#include "IBK_exception.h"
using namespace std;

namespace IBK {

void Flag::read(std::istream& in) {
    string name_;
    string state_;
	in >> name_ >> state_;
    if (!in)
        throw IBK_exception( IBK_FLAG_READ_ERROR,"[Flag::read]");
    if (state_ == "1" || string_nocase_compare(state_, "yes") || string_nocase_compare(state_, "true") ) {
    	state = true;
	}
	else if (state_ == "0" || string_nocase_compare(state_, "no") || string_nocase_compare(state_, "false") ){
    	state = false;
	}
	else
        throw IBK_exception( IBK::format_string(IBK_FLAG_INVALID_VALUE) % name_,"[Flag::read]" );
}
// ---------------------------------------------------------------------------

void Flag::write(std::ostream& out, unsigned int indent, unsigned int width) const {
    out << string(indent, ' ');
	out << setw(width) << left << name << " = ";
	if (state) 	out << "yes";
	else		out << "no";
}
// ---------------------------------------------------------------------------

}  // namespace IBK

