// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
#ifndef IBK_functiontrace_h
#define IBK_functiontrace_h

namespace IBK {

/// The function trace manager class, contains the current list of 
/// pointers to the function ID strings.
class FunctionTraceManagerT {
  public:
  	/// Default constructor
  	FunctionTraceManagerT() : m_currentIndex(0) {}
  	
  	/// Use this function to report the current trace to ERROR_MSG().
    void reportTrace();
    
  private:
    const char * m_functions[1000];
    unsigned int m_currentIndex; 

    friend class FunctionTrace2;
};

/// A global FunctionTraceManager object for the IBK library.
extern FunctionTraceManagerT 	GlobalFunctionTraceManager;
/// Global pointer to a function trace (make sure you only use it after
/// program initialization is complete!)
extern FunctionTraceManagerT  * FunctionTraceManager;

/// This is the function trace object which handles the
/// adding/removing of function traces to the function trace manager
/// automatically.
class FunctionTrace2 {
  public:
  	/// Constructor taking a pointer to a string constant which must be
  	/// valid throughout the lifetime of the FunctionTrace object.
    FunctionTrace2(const char * const fn_id) {
    	FunctionTraceManager->m_functions[FunctionTraceManager->m_currentIndex++] = fn_id;
    }

    ~FunctionTrace2() { --FunctionTraceManager->m_currentIndex; }
};

} // namespace IBK

#endif // IBK_functiontrace_h

