// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <iostream>
  #include <sstream>
  #include <fstream>
  #include <vector>
  #include <stdexcept>
#endif
#include "IBK_input.h"
#include "IBK_message_strings.h"
#include "IBK_format.h"
#include "IBK_stringutils.h"
#include "IBK_exception.h"
using namespace std;

namespace IBK {

void read_matrix(const std::string& filename, std::vector<double>& data, std::size_t& cols) {
    ifstream in(filename.c_str());
    if (!in)
        throw IBK_exception(format_string(IBK_FILE_OPEN_ERROR) % filename, "[IBK::read_matrix]");
    string line;
    // read first line to determine number of cols but skip empty lines and comments
    getline(in, line);
    while (in && trim(line).empty() || line[0]=='#')
        getline(in, line);

    stringstream strm(line);
    double tmp;
    std::vector<double> lvec;
    while (strm >> tmp)
        lvec.push_back(tmp);
    if (lvec.empty())
        throw IBK_exception(format_string(IBK_FILE_READ_ERROR) % filename, "[IBK::read_matrix]");
    cols = lvec.size();
    data = lvec;
    while (getline(in, line)) {
        if (trim(line).empty() || line[0]=='#') continue;
        stringstream lstrm(line);
        lvec.clear();
        while (lstrm >> tmp)
            lvec.push_back(tmp);
        if (lvec.size() != cols)
            throw IBK_exception(IBK_COLUMN_NUMBER_MISMATCH, "[IBK::read_matrix]");
        data.insert(data.end(), lvec.begin(), lvec.end());
    }
}
// ----------------------------------------------------------------------------

void read_spline(const std::string& filename, std::vector<double>& x, std::vector<double>& y) {
    ifstream in(filename.c_str());
    if (!in)
        throw IBK_exception(format_string(IBK_FILE_OPEN_ERROR) % filename, "[IBK::read_spline]");
    string line;
    x.clear();
    y.clear();
    while (getline(in, line)) {
        if (line.empty() || line.find_first_not_of(" \t")==string::npos || line[0]=='#') continue;
        stringstream lstrm(line);
        double x_,y_;
        if (lstrm >> x_ >> y_) {
            x.push_back(x_);
            y.push_back(y_);
        }
        else
        	throw IBK_exception(format_string(IBK_FILE_READ_ERROR) % filename, "[IBK::read_spline]");
    }
}
// ----------------------------------------------------------------------------

} // namespace IBK
