// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_input_h
#define IBK_input_h

#include <vector>
#include <string>

namespace IBK {

/// Reads a matrix from the file 'filename'.
/// The data is stored in the continuous vector 'data', the number of columns
/// is stored in 'cols'.
/// The functions throws an IBK::IBK_exception if an error occurs.
void read_matrix(const std::string& filename, std::vector<double>& data, std::size_t& cols);

/// Reads a spline from the file 'filename'.
/// The spline is stored using one data pair per line. The function reads data pairs until end
/// of file is encountered or no (complete) data pair can be read any longer.
/// The data is stored in the vectors 'x' and 'y' which will have the same size.
void read_spline(const std::string& filename, std::vector<double>& x, std::vector<double>& y);

} // namespace IBK

#endif // IBK_input_h

