// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <iostream>
  #include <sstream>
#endif
#include "IBK_keyword.h"
#include "IBK_format.h"
#include "IBK_message_strings.h"
#include "IBK_stringutils.h"
#include "IBK_exception.h"
using namespace std;

namespace IBK {

void Keyword::read(const std::string& keywordline) {
    stringstream strm(keywordline);
    strm >> category >> index >> name;
    if (!strm)
        throw IBK_exception(format_string(IBK_KEYWORD_LINE_READ_ERROR) % keywordline, "[Keyword::read]");
    // extract description
    size_t desc_start = keywordline.find("'");
    size_t desc_end = keywordline.rfind("'");
    if (desc_start!=string::npos && desc_end!=string::npos && desc_start!=desc_end)
        description = keywordline.substr(desc_start+1, desc_end-desc_start-1);
    // extract extra ids
    size_t pos = keywordline.find(name);
    pos += name.size() + 1;

    // if description is given, extract all characters in between first keyword and start of 
    // description
    if (desc_start!=string::npos) {
        other_ids = keywordline.substr(pos, desc_start-pos-1);
    }
    else {
        // otherwise take the rest of the string after the first keyword
        if (pos<keywordline.size())   other_ids = keywordline.substr(pos);
        else                          other_ids = "";
    }
    trim(other_ids);
}

std::string& trim_keyword(std::string& keyword) {
    if (keyword.empty())
        return keyword;
    size_t first = 0;
    size_t last = keyword.size()-1;
    if (keyword.at(0) == '[')  ++first;
    if (keyword.at(last) == ']')  --last;
    if (keyword.at(last) == '=')  --last;
    keyword = keyword.substr(first, last-first+1);
    return keyword;
};

} // namespace IBK
