// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <iostream>
  #include <fstream>
  #include <streambuf>
#endif
#include "IBK_logfile.h"
#include "IBK_messages.h"
#include "IBK_format.h"
#include "IBK_message_strings.h"
using namespace std;

namespace IBK {

// Constructor
Logfile::Logfile(ostream& original, const string& filename, std::ios_base::openmode ioflags)
    : m_original(NULL), m_buffer(NULL)
{
    m_filestream.open(filename.c_str(), ioflags);
    if (!m_filestream) {
		MSG( format_string(IBK_FILE_CREATE_ERROR) % filename, MSG_ERROR, "[LogFile::LogFile]");
        return;
    }
    m_original=&original;
    m_buffer=original.rdbuf();
    original.rdbuf(m_filestream.rdbuf());
};

// Destructor
Logfile::~Logfile() {
    if ((m_original!=NULL) && (m_buffer!=NULL))
        m_original->rdbuf(m_buffer);
}

// Turns logging off
void Logfile::redirect() {
    m_original->rdbuf(m_buffer);
}

// Turns logging on
void Logfile::redirect(const std::string& filename, std::ios_base::openmode ioflags) {
    // if there is already a file opened, close it first
    if (m_filestream)
        m_filestream.close();
    // open new logfile
    m_filestream.open(filename.c_str(), ioflags);
    if (!m_filestream) {
        MSG(format_string(IBK_FILE_CREATE_ERROR) % filename, MSG_ERROR, "[Logfile::redirect]");
		return;
    }
    if ((m_original!=NULL) && (m_buffer!=NULL))
        m_original->rdbuf(m_filestream.rdbuf());
}

} // namespace IBK

