// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2006 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_math_h
#define IBK_math_h

#include "IBK_configuration.h"

#include <string>
#include <cmath>

namespace IBK {

#ifdef IBK_ENABLE_SAFE_MATH

/// Safe variant for std::pow function taking a double exponent
double f_pow(double base, double e, std::string * calling_fn_name = NULL);

/// Safe variant for std::pow function taking an integer exponent
double f_pow(double base, int e, std::string * calling_fn_name = NULL);

/// Safe variant for std::pow(10,x) function 
double f_pow10(double e, std::string * calling_fn_name = NULL);

/// Safe variant for std::exp function 
double f_exp(double x, std::string * calling_fn_name = NULL);

/// Safe variant for std::log function
double f_log(double x, std::string * calling_fn_name = NULL);

/// Safe variant for std::log10 function
double f_log10(double x, std::string * calling_fn_name = NULL);

#else // IBK_ENABLE_SAFE_MATH

inline double f_pow(double base, double e, std::string * = NULL) { return std::pow(base,e); }
inline double f_pow(double base, int e, std::string * = NULL) { return std::pow(base,e); }
inline double f_pow10(double e, std::string * = NULL) {
#ifdef __GNUC__
#ifdef __MINGW32__
	return std::pow(10,e);
#else
	return pow10(e);
#endif
#else
	return std::pow(10,e);
#endif
}

inline double f_exp(double x, std::string * = NULL)   { return std::exp(x); }
inline double f_log(double x, std::string * = NULL)   { return std::log(x); }
inline double f_log10(double x, std::string * = NULL) { return std::log10(x); }

#endif // IBK_ENABLE_SAFE_MATH


} // namespace IBK

/// @file IBK_math.h
/// @brief Contains safe variants of standard C math functions.

#endif // IBK_math_h
