// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <iostream>
#endif

#include "IBK_messages.h"
#include "IBK_utilities.h"

namespace IBK {

MessageHandlerImpl::MessageHandlerImpl() {
	m_receiver = &m_defaultReceiver;
}

MessageHandlerImpl& MessageHandler() {
	static MessageHandlerImpl myMsgHandler;
    return myMsgHandler;
}

// implementation of default message handler function
// in this function we don't care about verbosity levels
void DefaultMessageReceiver::msg(const std::string& msg, msg_type_t t, const char * func_id, int /* verbose_level */) {
	switch (t) {
    	case MSG_PROGRESS:
#ifdef IBK_INCLUDE_FUNCID
            if (func_id != NULL)
                std::cout << func_id << "  " << msg;
            else
#endif // IBK_INCLUDE_FUNCID
                std::cout << msg;;
            std::cout.flush();
            break;
            
        case MSG_WARNING :
			IBK::set_console_text_color(IBK::CF_BRIGHT_YELLOW);
#ifdef IBK_INCLUDE_FUNCID
        	if (func_id != NULL)
            	std::cout << func_id << "  " << msg << std::endl;
        	else
#endif // IBK_INCLUDE_FUNCID
            	std::cout << msg << std::endl;
			IBK::set_console_text_color(IBK::CF_GREY);
		 	break;

		case MSG_ERROR :
			IBK::set_console_text_color(IBK::CF_BRIGHT_RED);
        	if (func_id != NULL)
            	std::cerr << func_id << "  " << msg << std::endl;
        	else
            	std::cerr << msg << std::endl;
			IBK::set_console_text_color(IBK::CF_GREY);
		 	break;

        case MSG_DEBUG :
			IBK::set_console_text_color(IBK::CF_BRIGHT_MAGENTA);
            if (func_id != NULL)
                std::clog << func_id << "  " << msg << std::endl;
            else
                std::clog << msg << std::endl;
			IBK::set_console_text_color(IBK::CF_GREY);
            break;
        default : ; // do nothing
    }
}

} // namespace IBK

