// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <iostream>
  #include <iomanip>
  #include <iterator>
#endif
#include "IBK_output.h"
#include "IBK_keywordlistmanager.h"
using namespace std;

namespace IBK {

ostream& write_vector(ostream& out, const string& timestr, const vector<double>& vec) {
    out << left << timestr << '\t';
    copy(vec.begin(), vec.end(), ostream_iterator<double>(out, "\t") );
    return out;
}
// ----------------------------------------------------------------------------

ostream& write_matrix(ostream& out, const IBK::matrix<double>& mat, unsigned int spacing) {
    for (unsigned int j=0; j<mat.rows(); ++j) {
        for (unsigned int i=0; i<mat.cols(); ++i)
            if (spacing == 0)   out << mat.at(i,j) << '\t';
            else                out << setw(spacing) << mat.at(i,j) << '\t';
        out << '\n';
    }
    return out;
}
// ----------------------------------------------------------------------------

std::ostream& write_keyword(std::ostream& out, const std::string& istr, unsigned int len, const char * const kw) {
	return (out << istr << setw(len) << left << kw << " = ");
}
// ----------------------------------------------------------------------------

std::ostream& write_flag(std::ostream& out, const std::string& istr, unsigned int len,
	const char * const kw_category, int index, bool on)
{
	out << istr << setw(len) << left << KeywordList->get_name(kw_category, index) << " = " << (on ? "yes\n" : "no\n");
	return out;
}
// ----------------------------------------------------------------------------


std::ostream& write_spline(std::ostream& out, const std::string& istr, unsigned int len,
	const IBK::LinearSpline & sp, const std::string& fn_name)
{
	if (!sp.empty()) {
    	write_keyword(out, istr, len, "FUNCTION") << fn_name << endl;
        sp.write(out, static_cast<unsigned int>(istr.size())+4);
    }
    return out;
}
//---------------------------------------------------------------------------

} // namespace IBK
