// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_output_h
#define IBK_output_h

#include <iosfwd>
#include <string>
#include <vector>
#include "IBK_matrix.h"
#include "IBK_linearspline.h"

namespace IBK {

/// Writes data into an output file for a time point given in timestr. Between the timestr and the 
/// vector data a tab character is placed and also between all columns of the vector vec. A 
/// typical output line for a vector of 3 elements and a timestring of 1.34 would look like:
///    '1.34    12.33    43.0    33.0'
std::ostream& write_vector(std::ostream& out, const std::string& timestr, const std::vector<double>& vec);

/// Writes a matrix output.
/// The matrix is dumped in the stream in columns separated with tabulators.
std::ostream& write_matrix(std::ostream& out, const IBK::matrix<double>& mat, unsigned int spacing=0);

/// Writes a formatted keyword (which is equivalent to 'out << istr << setw(len) << left << keyword << " = "').
std::ostream& write_keyword(std::ostream& out, const std::string& istr, unsigned int len, const char * const kw);

/// Writes a formatted flag (which is equivalent to 'out << istr << setw(len) << left << flag << " = " << "yes/no" ').
std::ostream& write_flag(std::ostream& out, const std::string& istr, unsigned int len, const char * const kw_category, int index, bool on);

/// Writes the formatted keyword 'FUNCTION' with the lengths len and in the next two lines
/// the x and y values of the spline sp.
std::ostream& write_spline(std::ostream& out, const std::string& istr, unsigned int len,
	const IBK::LinearSpline & sp, const std::string& fn_name);


/// @file IBK_output.h
/// @brief Contains the declarations for the output functions in the IBK library.

/// @example output_example.cpp
/// This example demonstrates the use of the IBK lib output functions.

} // namespace IBK

#endif  // IBK_output_h
