// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
//
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <sstream>
  #include <iomanip>
  #include <limits>
  #include <cmath>
  #include <iostream>
#endif
#include "IBK_stopwatch.h"
using namespace std;

namespace IBK {

clock_t StopWatch::start() {
    start_=clock();
    stop_=0;
    return start_;
}

double StopWatch::stop() {
    stop_ = clock();
    return (stop_-start_)*1000.0 / CLOCKS_PER_SEC;
}

double StopWatch::difference() const {
    if (stop_!=0) {
    	return (stop_-start_)*1000.0 / CLOCKS_PER_SEC;
	}
    else {
		clock_t c = clock();
		double ct = c;
		return (ct-double(start_))*1000.0 / CLOCKS_PER_SEC;
	}
}

const string StopWatch::diff_str() const {
    stringstream strm;
    double diff=difference();
    if (diff<1000)
        strm << static_cast<int>(diff) << " ms";
    else if (diff < 60000)
        strm << fixed << setprecision(3) << diff/1000.0 << " s";
    else if (diff < 3600000)
        strm << fixed << diff/60000.0 << " min";
	else
        strm << fixed << diff/3600000.0 << " h";
    return strm.str();
}

const string StopWatch::diff_str(size_t width) const {
    stringstream strm;
    double diff=difference();
    if (diff<1000)
        strm << setw(static_cast<int>(width)-3) << static_cast<int>(diff) << " ms";
    else if (diff < 60000)
        strm << setw(static_cast<int>(width)-2) << fixed << setprecision(3) << diff/1000.0 << " s";
    else if (diff < 3600000)
        strm << setw(static_cast<int>(width)-4) << fixed << setprecision(3) << diff/60000.0 << " min";
    else
        strm << setw(static_cast<int>(width)-2) << fixed << setprecision(3) << diff/3600000.0 << " h";
    return strm.str();
}

}  // namespace IBK
