// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <stdexcept>
#endif
#include "IBK_tridiagmatrix.h"
using namespace std;

namespace IBK {

template <>
unsigned int tridiag_matrix<double>::solve_equation_system(double * rhs) {
    double * d = &data_.front()  + size_;		// get direct access to storage memory
    double * b = d - size_;						// b holds now the lower band (b[0] not used)
    double * a = d + size_;						// c holds the upper band (c[m_size-1] not used)
    int i=1;									// leave out first element (i=0)
    for (; i<static_cast<int>(size_); ++i) {
        if (d[i-1]==0.0)	return 1;	// Zero element in main diagonal
		double fact = b[i]/d[i-1];
        d[i] -= fact*a[i-1];
        if (d[i]==0.0)		return 2;	// Matrix is singular or becomes singular by transformation
        rhs[i] -= fact*rhs[i-1];
    }
    rhs[static_cast<int>(size_)-1] /= d[static_cast<int>(size_)-1];
    for (i=static_cast<int>(size_)-2; i>=0; --i) {
        rhs[i] -= a[i]*rhs[i+1];
        rhs[i] /= d[i];
    }
    return 0; // all ok
};
// ---------------------------------------------------------------------------

} // namespace IBK

