// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_types_h
#define IBK_types_h

#include <functional>

namespace IBK {

/// Abstract base class for the function objects taking one double as argument.
class DblFunction : public std::unary_function<double, double> {
public:
	DblFunction() {}
	/// Virtual destructor to ensure destruction of data members of derived classes.
	virtual ~DblFunction() {}
	/// Actual implementation of the function encapsulated in this function object.
	/// You have to reimplement this function in your derived class!
	virtual double operator() (double x) const = 0;
};

}  // namespace IBK

/// @file IBK_types.h
/// @brief Contains the type definitions used throughout the IBK library.

#endif  // IBK_types_h

