// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <stdexcept>
  #include <iostream>
#endif
#include "IBK_unit.h"
#include "IBK_unitdata.h"
#include "IBK_unitlistmanager.h"
#include "IBK_stringutils.h"
#include "IBK_exception.h"
using namespace std;

namespace IBK {

Unit::Unit(const std::string& name) {
    if (UnitList==NULL)  throw IBK_exception("Unitlist not initialised!","[Unit::Unit]");
    else {
        const UnitData* unit( UnitList->retrieve(name) );
        if (unit==NULL)  throw IBK_exception( "Unit '"+name+"' not found!","[Unit::Unit]");
        id_=unit->id();
    };
}
// ---------------------------------------------------------------------------

Unit::Unit(unsigned int unitid) {
    if (UnitList==NULL)  throw IBK_exception("Unitlist not initialised!","[Unit::Unit]");
    else if (unitid>=UnitList->size())
        throw IBK_exception("Unit index "+val2string(unitid)+
            " out of range 0.."+val2string(static_cast<int>(UnitList->size())-1)+"!","[Unit::Unit]");
    id_=unitid;
}
// ---------------------------------------------------------------------------

void Unit::set(unsigned int unitid) {
    if (UnitList==NULL)  throw IBK_exception("Unitlist not initialised!","[Unit::set]");
    if (unitid>=UnitList->size())
        throw IBK_exception("Unit index "+val2string(unitid)+
            " out of range 0.."+val2string(static_cast<int>(UnitList->size())-1)+"!","[Unit::set]");
    id_=unitid;
}
// ---------------------------------------------------------------------------

void  Unit::set(const std::string& name) {
    if (UnitList==NULL)  throw IBK_exception("Unitlist not initialised!","[Unit::set]");
    else {
        const UnitData* unit( UnitList->retrieve(name) );
        if (unit==NULL)  throw IBK_exception( "Unit '"+name+"' not found!","[Unit::set]");
        id_=unit->id();
    };
};
// ---------------------------------------------------------------------------

const string& Unit::name() const {
    return UnitList->retrieve(id_)->name();
}
// ---------------------------------------------------------------------------

unsigned int Unit::base_id() const {
    return UnitList->retrieve(id_)->base_id();
}
// ---------------------------------------------------------------------------

void Unit::relate_to(unsigned int unitid, double& fact, unsigned int& op) {
    if (unitid==id_) {
        fact=1;
        op=UnitListManager::OP_NONE;
        return;
    }
    const UnitData* from( UnitList->retrieve( id_ ) );
    const UnitData* to( UnitList->retrieve( unitid ) );
    UnitList->relate_units(from, to, fact, op);
}
// ---------------------------------------------------------------------------

void Unit::relate_to(const string& name, double& fact, unsigned int& op) {
    unsigned int to_id( UnitList->retrieve(name)->id() );
    relate_to(to_id, fact, op);
}
// ---------------------------------------------------------------------------

void Unit::relate_to(Unit unit, double& fact, unsigned int& op) {
    relate_to(unit.id_, fact, op);
}
// ---------------------------------------------------------------------------

istream& operator>>(istream& in, Unit& unit) {
    string uname;
    in >> uname;
    if (!in) return in;
    unit = Unit(uname);
    return in;
}
// ---------------------------------------------------------------------------

ostream& operator<<(ostream& out, const Unit& unit) {
    return out << unit.name();
}
// ---------------------------------------------------------------------------

}  // namespace IBK

