// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#include "IBK_configuration.h"

#ifdef __BORLANDC__
  #include "IBK_prehead.h"
  #pragma hdrstop
#else
  #include <fstream>
  #include <sstream>
  #include <iostream>
#endif
#include "IBK_utilities.h"
#include "IBK_stringutils.h"
#include "IBK_messages.h"
#include "IBK_format.h"
using namespace std;

#if defined (IBK_ENABLE_COLORED_CONSOLE) && (defined(_MSC_VER) || defined(__BORLANDC__) || defined(__MINGW32__))
  #include <windows.h>
  #include <cstdio>
#endif

namespace IBK {

extern const char * const TERMINAL_CODES[16];

bool default_init(UnitListManager& ulist, KeywordListManager& klist,
	const std::string& executable, const char * const kw_data,
    std::string& errorMsg)
{
	string FUNC_ID = "[IBK::default_init]  ";
	string path, solver_file;
	IBK::split_filename(executable, solver_file, path);

	// ****** Initialisation ******
	// Unit list
	UnitList = &ulist;                      // assign global pointer (required)
    ifstream in( (path + "units.txt").c_str());   // can I open the file?
    if (in) {
    	// try to read the units.txt
    	if (!UnitList->read(in)) {
			errorMsg = FUNC_ID + "Cannot read unit list from file '"+path+"units.txt'!";
            return false;
        }
    }
    else {
    	// read the default unit list instead
    	if (!UnitList->read(DEFAULT_UNITS)) {
			errorMsg = FUNC_ID + "Error reading internal unit list!";
            return false;
        }
    }

	// Keyword list
	KeywordList = &klist;       // assign global pointer (required)
    if (kw_data == NULL) {
    	ifstream kwfile((path + "keywords.txt").c_str());
        // try reading keywordlist from file
        if (!KeywordList->read(kwfile) || KeywordList->empty()) {
            errorMsg = FUNC_ID + "Cannot read keyword list from file '"+path+"keywords.txt'!";
            return false;
        }
    }
    else {
    	stringstream strm(kw_data);
        if (!KeywordList->read(strm) || KeywordList->empty()) {
            errorMsg = FUNC_ID + "Cannot read keyword list from keyword list data string!";
            return false;
        }
    }
	return true;
}

void set_console_text_color(console_color_t c) {
#if defined(IBK_ENABLE_COLORED_CONSOLE) && (defined(_MSC_VER) || defined(__BORLANDC__) || defined(__MINGW32__))
    HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);  // Get handle to standard output
	SetConsoleTextAttribute(hConsole,static_cast<WORD>(c));
#else
	//int i = static_cast<int>(c) % 16;
	//std::cerr << TERMINAL_CODES[i] << i << " \033[22;31m";
	//std::cout << "\033[22;31m";
#endif
}

extern const char * const TERMINAL_CODES[16] = 
{
"\033[22;30m", // black
"\033[22;31m", // red
"\033[22;32m", // green
"\033[22;33m", // brown
"\033[22;34m", // blue
"\033[22;35m", // magenta
"\033[22;36m", // cyan
"\033[22;37m", // gray
"\033[01;30m", // dark gray
"\033[01;31m", // light red
"\033[01;32m", // light green
"\033[01;33m", // yellow
"\033[01;34m", // light blue
"\033[01;35m", // light magenta
"\033[01;36m", // light cyan
"\033[01;37m"  // white
};

}  // namespace IBK

