// IBK Library - Collection of classes, functions and algorithms
//               for development of numeric simulations
// 
// Copyright (C) 2003 Andreas Nicolai, Andreas.Nicolai@gmx.net
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

#ifndef IBK_utilities_h
#define IBK_utilities_h

#include <string>
#include <iosfwd>
#include <stdexcept>
#include "IBK_unitlistmanager.h"
#include "IBK_keywordlistmanager.h"
#include "IBK_format.h"

namespace IBK {

/// This function is used for the standard initialisation of the IBK library. 
/// You need to pass a unit list and keyword list object, that MUST EXIST over 
/// the whole lifetime of the program and also the executable name (including 
/// the full path to it - can be retrieved from the first command line argument).
/// The function reads in the unit list from the file units.txt if it exists in 
/// the executable directory, otherwise the default unit list is read. 
/// Then it reads the keywords.txt into the keywords list. The function also assignes
/// the global unitlist and keywordlist pointer to the objects you passed to the 
/// function. NO MEMORY IS ALLOCATED HERE!
/// If there should be an error (no keywords.txt or something like that) the 
/// function returns false.
bool default_init(UnitListManager& ulist, KeywordListManager& klist,
	const std::string& executable, const char * const kw_data,
    std::string& errorMsg);

/// Foreground and background colors.
enum console_color_t {
	CF_BLACK 			= 0x00,
	CF_BLUE				= 0x01,
	CF_GREEN			= 0x02,
	CF_CYAN 			= 0x03,
	CF_RED				= 0x04,
	CF_MAGENTA			= 0x05,
	CF_YELLOW			= 0x06,
	CF_GREY				= 0x07,
	CF_DARK_GREY		= 0x08,
	CF_BRIGHT_BLUE		= 0x09,
	CF_BRIGHT_GREEN		= 0x0A,
	CF_BRIGHT_CYAN		= 0x0B,
	CF_BRIGHT_RED 		= 0x0C,
	CF_BRIGHT_MAGENTA	= 0x0D,
	CF_BRIGHT_YELLOW	= 0x0E,
	CF_WHITE			= 0x0F,
	CB_BLACK 			= 0x00,
	CB_BLUE				= 0x10,
	CB_GREEN			= 0x20,
	CB_RED 				= 0x30,
	CB_MAGENTA			= 0x40,
	CB_CYAN				= 0x50,
	CB_YELLOW			= 0x60,
	CB_GREY				= 0x70,
	CB_DARK_GREY		= 0x80,
	CB_BRIGHT_BLUE		= 0x90,
	CB_BRIGHT_GREEN		= 0xA0,
	CB_BRIGHT_RED 		= 0xB0,
	CB_BRIGHT_MAGENTA	= 0xC0,
	CB_BRIGHT_CYAN		= 0xD0,
	CB_BRIGHT_YELLOW	= 0xE0,
	CB_WHITE			= 0xF0
};

/// @brief Sets the color of the console text used for the next output.
///
/// The color 'c' is a logical or of the foreground and background colors
/// defined in the console_color_t enum. To set a bright yellow text color
/// with a blue background use:
/// @code
/// set_console_text_color(CF_BRIGHT_YELLOW | CB_BLUE);
/// @endcode
void set_console_text_color(console_color_t c);

}  // namespace IBK


/// @file IBK_utilities.h
/// @brief Contains small helper functions.

#endif  // IBK_utilities_h
