// assembly.cpp : Defines the entry point for the console application.
//
//#include "stdafx.h"
#include <iostream> 
//#include <cstdlib> 
#include <fstream> 
#include <string> 
#include <stdlib.h>
#include <math.h>

using namespace std;

void PnETCN(void);
void SetTableau (void);
void AllocateMo (void);
void AllocateYr (void);
void AtmEnviron (void);
void CanopyInteraction (void);
void CNDOYOut (void);
void CNInit (void);
void CNNewYear (void);
void CNTrans (void);
void Decomp (void);
void Phenology (int);
void Photosyn (void);
void ReadDay (void);
void Scenario (void);
void SoilHydrology (void);

double ZMax (double, double);
double ZMin (double, double);
void InitPools(void);
void CNYrTitles(void);
double ZCos(double);
void Leach2(void);
void EquilibriumCalc(int);

void Saturate (int );
void SimplifyProblem (int );

void TempCorrection (void);
void FirstGuess (void);

void CalcSpeciesConc (void);
void CalcActivityCoeff (void);

void CalcResidual (double & maxerror, int & Maxcomp);
void JacobianAA (void);
void GAUSS (void);
void SUpdate (void);
void CheckSolidPhase (bool &, int);

double Davies(int);

void CNFinalOut(void);

void CNYearOut (int);
void UpdateData(void);

/*data dictionary
Amax = maximum net photosynthesis for broad-leaved, decidous forests, same as NetPsnmax from Aber (1992). Wu
            calculated as AmaxA + AmaxB*FolNCon, not valid for needle-leaved evergreen species. nmol CO2/g foliage/sec
AmaxFrac = the adjustment of fraction to Amax before calculating GrossAmax. Wu
AmaxA = intercept coefficient to calculate Amax. Wu. Read from *.veg file.
AmaxB = slope coefficient to calculate Amax. Wu. Read from *.veg file.
BaseFolResp = basal respiration of foliage
BaseFolRespFrac = fraction of basal respiration of foliage to maximum net photosynthesis
DayLength = day length (seconds) -- Aber et al., 1992,  calculated from Lat (latitude) and DOY (day of year, midpoint of each month) 
                   using equations from Swift (1976) and Sellers (1965).
DayResp = respiration during day
DTemp = between 0 and 1, express the effects of temperature on maximum gross photosynthesis to get the actual photosynthesis 
DWater = between 0 and 1, express the effects of water availability on maximum gross photosynthesis to get the actual
               photosynthesis, soil water stress on stomatal closure
DVPD = between 0 and 1, express the effects of vapor pressure deficit on maximum gross photosynthesis to get the actual 
            photosynthesis 
FolNCon = foliar nitrogen content, percent nitrogen in foliage (dry weight). If there is 2% of nitrogen in the leaves,
                FolNCon is 2, not 0.02.
GrossAmax = maximum gross photosynthesis, same as GrossPsnmax from Aber (1992).
NightResp = respiration during night
PsnTMax = the upper temperature limit for photosynthesis. -- Wu, in the environment where temperature exceeds this value, 
                 it is not going to be photosynthesis. calculated
PsnTMin = the lower temperature limit for photosynthesis. -- Wu, in the environment where temperature is lower than this value, 
                 it is not going to be photosynthesis. read from *.veg file
PsnTOpt = optimum temperature for photosynthesis. -- Wu. read from *.veg file
RespQ10 = Q10 factor
Tavg = mean daily temperature -- Aber et al., 1992, (Tmax + Tmin)/2
Tday = mean daytime temperature -- Aber et al., 1992, (Tavg+Tmax)/2
Tmax = maximum temperature -- Aber et al., 1992, read from climate file "HB3Xbgc.csv"
Tmin = mininum temperature -- Aber et al., 1992, read from climate file "HB3Xbgc.csv"
Tnight = mean nighttime temperature -- Aber et al., 1992
Tday and Tnight are used in calculations of leaf respiration and photosynthesis
VPD = vapor pressure deficit (KPa) -- Aber et al., 1992. The difference between saturated vapor pressure at mean daily temperature
          and minimum temperature using the equation Murray(1967).
WUE = water use efficiency in mg C fixed per g H2O transpired -- Aber et al., 1992.
WUEConst = a constant -- Aber et al., 1992. according to Baldocchi et. al., 1987: for deciduous forests, it is 10.9.

*/


 string SoilFile, StrmFile, DepFile, MonthFile;
 string opClimFile, wetfile, dryfile;
 string WaterFile; // //Afshin Pourmokhtarian added 11/11/09
 string NPPFile; // //Afshin Pourmokhtarian added 11/11/09

double RootCAdd, GDDWoodEff, OldFolMass, PlantN, RootAllocCMo; // add after compiling debug1 "allocateMo()"
double NR; // add after compiling "CNInit.cpp"
int NElem, PrintInt = 1, nyears; // add after compiling "CNInit.cpp"
bool Carbon, Nitrogen; // add after compiling "CNInit.cpp"
string Title; // add after compiling "CNInit.cpp"
int Countit, ThisYear, iday; // add after ReadDay.cpp
double FolC, FolN, TotalN ; // add after CNYearOut.cpp
int DropSwitch, FolMRespYr; // add after CNNewYear.cpp
double GrossNImmobYr; // add after CNNewYear.cpp
double MaxFolMassYr; // add after CNNewYear.cpp
double DepNH4, DepNO3; // add after scenario.cpp
int CCRamp; // add after AtmEnviron.cpp
double decl, GDD, isc, LatRad, ninx1, pi, r, z, z2; // add after AtmEnviron.cpp
double FolMassNew; // add after phenology.cpp
//int GddFolEnd; // add after phenology.cpp
double Amax, BaseFolResp, DLight, Il, LayerGrossPsn, LayerGrossPsnRate, LayerNetPsn, LayerResp, LightEff, PARTot, SLWLayer, 
       ten9, CanopyTrans, GrossAmax; // add after photosyn.cpp
double depanions, depcations, folexchange; // add after CanopyInteraction.cpp
double CanopyGrossPsnMG, FastFlow, FracSnowMelt, precrem, SnowFrac, SnowMelt, Tavew, TotSoilMoistEff, Transd, waterin, waterind,
       wday, WFactor, WUE; // add after SoilHydrology.cpp
// int iElH2O; // add after SoilHydrology.cpp
double FolNLoss, LitterFallN, Remove, Retrans, RootTurnover, WoodMassLoss; // add after CNTrans.cpp
double DOCmin, GrossNImmob, KhoAct, NImmobA, NImmobB, NReten, RootNSinkStr, SoilPctN, WMult; // add after Decomp.cpp
int LinkList[100][100], Maxcomp, Iter; // after Leach2, change LinkList from double to integer LinkList[4][21]
double sumaqua, sumsurf, maxerror; // after Leach2
string PName[100]; // after Saturate PName[6]
double RoundOffLimit; // after Saturate
bool Used; // after SimplifyProblem
int NSpCalc; // after SimplifyProblem change from double to integer
double IStr; // after FirstGuess
double Alactivity, cPLOGK, Hactivity, IAP; // after CheckSolidPhase
double deA, deB, dielectric, SatIndex, SqrtI; // after Davies
int istop; // afterCNDOYOut
double MDrainMgL ; // afterDOYOut
int Nj, spID; //after SetTableau
double avgPCBM, EnvMaxFol, SppMaxFol, TotalC; // after AllocateYr FolN, NR, 
int PrintLevel; // after CNYrTitle change from double to integer
int outdatx[1201]; // after CNInit change from double to integer
int LinkCoeff[100][100]; //LinkCoeff[4][21];
//double EFCN[21], ELCN[21, EPCN[21], EWeather[21]; // after UpdateData
double ELCN[100]; //ELCN[21];
double FolAl, FolCa, FolCl, FolF, FolK, FolMg, FolNa, FolP, FolS, FolSi, LitAl, LitCa, LitCl, LitF, LitK, LitMg, LitNa, LitP, LitS, LitSi,
       PlantAl, PlantCa, PlantCl, PlantF, PlantK, PlantMg, PlantNa, PlantP, PlantS, PlantSi, WInputAl, WInputCa, WInputCl, WInputF,
	   WInputK, WInputMg, WInputNa, WInputP, WInputS, WInputSi; //after UpdateData

double AgHarv, AllocYr, AmaxA, AmaxB, AmaxFrac, 
      AvgDWater, BaseFolRespFrac, BaseYear, BiomLossFrac, BudC, BudN, // 6 variables
	  CanopyGrossPsn, CanopyGrossPsnAct, CanopyGrossPsnActMo, CanopyNetPsn, CFracBiomass,
	  ChronH2OAdd, ChronH2OStart, ChronNAdd, ChronNEnd, ChronNStart,
      DayLength, DayResp, DaySpan, DeadWood, DeadWoodM, 
	  DeadWoodN, DelPrec, DelSolRad, delta, DeltaET, 
	  DelTMax, DelTMin, Drainage, DTemp, DVPD, 
	  DVPD1, DVPD2, DWater, DwaterIx, Dwatertot,
	  emean, es, Evap, f, FastFlowFrac, 
	  FertYear, FLPctN, FolGRespMo, FolGRespYr, FolHarv,
	  FolHarvN, FolLitM, FolLitN, FolMass, FolMassInc, 
	  FolMassMax, FolMassMin, FolNCon, FolNConRange, FolNRetrans,
      FolProdCMo, FolProdCYr, FolRelGrowMax, FolReten, FracDrain,
      GCE, GCEStat, GDDFol, GDDFolEff, GDDFolEnd,
	  GDDFolStart, GDDFolTot, GDDTot, GDDWoodEnd, GDDWoodStart, // add GDDFolEnd in the debug_read.cpp
	  graph, GRespFrac,  GrossNMin, //GrossNlmmob, GrossNlmmobYr,
	  GrossNMinYr, h, HalfSat, Harvlntens[100], HarvYear[100], //Harvlntens[6], HarvYear[6],
	  HarvYet, HOM, HON, hr, 
	  io, Iomax, ixFolMass, Kho, 
	  kkk, LAI, LastDay, Lat, LightEffMin, 
	  littertotal, LRatio, MaxNStore, MeanAnnFolMass, MeanAnnT, 
	  MeanGCEMeas, MeanGCEPred, MeanGCEStat, MeanNetPsnPred, MeanSoilMoistEff, 
	  MinWoodFolRatio, ndays, NDrain, NDrainMgL, NDrainYr,
	  NEP, NetCBal, NetNitr, NetNitrYr, NetNMin, 
	  NetNMinLastYr, NetNMinYr, NetPsnMo, NewYear, 
	  NFert, NH4, NH4Dep, NH4Up, NightLength,
	  NightResp, NMinH, NO3, //NlmmobA, NlmmobB, 
	  NO3Dep, NO3Up, NPPFolYr, NPPRootYr, NPPWoodYr,
	  NRatio, NRatioNit, nsites, Ntot, ntypes,
	  outdaty[100][1201], outn, OutptYrFrom, //outdaty[9][1201],
	  OutptYrTo, PAR, PlantC, PlantCReserveFrac, PlantNDemand, 
	  PlantNUptake, PlantNUptakeYr, PosCBalMass, PosCBalMassIx, PosCBalMassTot,
	  PotTransd, Prec, PrecIntFrac, Printlnt, 
	  PsnTMax, PsnTMin, PsnTOpt, RealYear, RemFrac[100], //RemFrac[6],
	  RemoveFrac, RepQ10, RespQ10, RLPctN, RootAllocA,  
	  RootAllocB, RootC, RootGRespMo, RootGRespYr, RootLitM,
	  RootLitN, RootMass, RootMassInc, RootMassN, //4 
	  RootMRespFrac, RootMRespMo, RootMRespYr, RootNSinkEff, RootProdCMo,
	  RootProdCYr, RootTurnoverA, RootTurnoverB, RootTurnoverC, ScenFile,
	  SenescStart, SLossFrac[100], SLWdel, SLWmax, SnowPack, //SLossFrac[6],
	  SnowPackNH4, SnowPackNO3, SoilCN, SoilDecResp, SoilDecRespYr, 
	  SoilMoistFact, //add when debugging debug_read.cpp
	  SoilRespA, SoilRespB, SoilRespMo, SoilRespYr, solrad,
	  stype, T, Tave, Tday, tEffSoil,  
	  ThisMonth,  Tmax, Tmin, TminLastMo, 
	  TMult, Tnight, TotalLitterM, TotalLitterMYr, TotalLitterN,
	  TotalLitterNYr, TotalM, TotalPlantN, TotAnnFolMass, TotDepNYr,
	  TotDrain, TotEvap, TotGCEMeas, TotGCEPred, TotGCEStat, 
	  TotNetPsnPred, TotPrec, TotPsn, TotT, totTrans,
	  Trans, Valid1, Valid2, Valid3, Valid4, 
	  VPD, vtype, Water, WaterLost, WetDepNh4Con, 
	  WetDepNO3Con, WHC, WLPctN, WoodC, WoodDecResp, 
	  WoodDecRespYr, WoodGRespMo, WoodGRespYr, WoodHarv, WoodHarvN,
	  WoodLitCLoss, WoodLitLossRate, WoodLitM, WoodLitN, WoodLitTot, WoodMass,   //6 variables
	  WoodMassInc, WoodMassN, WoodMRespA, WoodMRespMo, WoodMRespYr, WoodProdCMo, //6 variables
	  WoodProdCYr, WoodTransM, WoodTransN, WoodTurnover, WUEConst, 
	  XMax, XMin, YMax[100], YMin[100]; //YMax[9], YMin[9];
int DOY, nextday, ix;
int  countdays;

//Afshin Pourmokhtarian 06/23/10

			double SoilNa, SoilMg, SoilAl, SoilK, SoilCa, SoilNO3, SoilSO4;

//Afshin Pourmokhtarian 06/23/10


/*char ClimFile, DryDepFile, FileDir, OpClimFile, OpSiteFile, 
     OpTypeFile, SiteNmae, TypeName, WetDepFile, WetDryFile;
char[8] varName;
string Sitefile[8], sitelst[20], typfile[8], typlst[20]; */

string FileDir, OutDir, FileVeg, TypeFile;
string EName[100]; //EName[81];

bool icycle;

// global variables in soil chemistry
/*double aij[9][21], AnionNeutCap, BaseSaturation, bij[81][21], CConc[80],
      CEC, Charge[80], CompOrder[20], DeltaH[80], Gamma[80], 
	  initAlc, initAlp, initAls, initSolid[10], inittime, 
	  LStr, lter, Jacobian[20][20], 
	  LOGK[80], NComp, NList, NPhase, 
	  NSpecies, OrganicAl, Paij[5][20], PartCoeff[18], PDeltaH[5], 
	  PLOGK[5], PMass[5], Ptype[5], Residual[20], SConc[80],
	  SiteDOC, SO4AdCapacity, SoilAlCaRatio, SoilDensity, SoilMass,
	  SoilpH, SpeciesOrder[80], taij[80][20], tbij[80][20], tDeltaH[80],
	  TempK, tLOGK[80], Tolerance, Totals[20], totSoilWater,
	  tPLOGK[5];
int ActCorr[80], Alindex, ALPPT, CO3index, Ctype[20], 
    DOCindex, iSing, NFirstS, NSolve, Orgindex, 
	Sptype[80];*/

double  AnionNeutCap, BaseSaturation, 
      CEC, DeltaH[100], Gamma[100], //, Charge[81], DeltaH[81], Gamma[81], 
	   initSolid[100], inittime, //initSolid[11],
	  LStr, lter, Jacobian[100][100], //Jacobian[21][21], 
	  LOGK[81],  NList, NPhase, 
	  OrganicAl, PartCoeff[100], PDeltaH[100], //Paij[6][21], PartCoeff[19], PDeltaH[6], 
	  PLOGK[100], PMass[100],  Residual[100], //PLOGK[6], PMass[6], Ptype[6], Residual[21], 
	  SiteDOC, SO4AdCapacity, SoilAlCaRatio, SoilDensity, SoilMass,
	  SoilpH,  tDeltaH[100], //SpeciesOrder[81],  tDeltaH[81],
	  TempK, tLOGK[100], Tolerance, Totals[100], totSoilWater, //tLOGK[81], Totals[21], 
	  tPLOGK[100]; //PLOGK[6]

int aij[100][100], taij[100][100], bij[100][100], Paij[100][100], tbij[100][100], Charge[100], SpeciesOrder[100]; //aij[81][21], taij[81][21], bij[81][21], tbij[81][21]; 

int CompOrder[100]; //CompOrder[21];
double CConc[100], SConc[100]; //CConc[81], SConc[81];
int ActCorr[100], Alindex, ALPPT, CO3index, Ctype[100], // ActCorr[81], Ctype[21]
    DOCindex, iSing, NFirstS, NSolve, Orgindex, 
	Sptype[100], Ptype[100], NComp, NSpecies, initAlc, initAlp, initAls; //Sptype[81]

// char EName[80], PName[5];
bool isAl;

//add from debug_read
int AgFrom, AgRem, AgTo, CCYrFrom, CCYrTo, Check1Value, FertDOYEnd, FertDOYStart, FertYrEnd, FertYrStart, FolRegen, harvno, RunModelFrom, RunModelTo;
double DelPar, DelWUE, FertNH4, FertNO3, HarvIntens[100]; // HarvIntens[6];

const double RoundOfLimit = 0.1;

const int stAqueous = 0, stSurface = 1; //species type

const int ctFloat=1, ctAnalog=2, ctSubstituted=3, ctChargeBalance=4, ctFixed=5, ctSurfPot=15; //component type

const int ptUnlimited=-1, ptSaturated=1, ptUnsaturated=2, ptNotAvailable=3; //phase types

const int acCone=1, acDavies=2, acChargeFrac=5; //activity correction type

const double Rcon = 1.987; //cal/deg-mol

const int rxtStream = 1, rxtSoil = 0;

// global variables in BGC portion of linked model
bool iETreat;

const int iAlOrg=16,iANC=17, iElAl=5, iElCa=7, iElCl=9,
          iElDIC=12, iElDOC=2, iElF=13, iElH=1, iElH20=0,
          iElK=6, iElMg=4, iElNa=3, iElNH4=8, iElNO3=10, 
          iElPO4=14, iElSi=15, iElSO4=11;

/*double AvgSoil[20], AvgStream[40], AvSolid[20], CO2Prod, DHO,
      DOCPart, DOCProd, EDep[20], EDryDepRatio[20], EFastFlow[20],
	  EFCN[20], EFolEx[20], EFolUp[20], EHumus[20], ElCN[20],
	  Eleach[20], ELit[20], EMin[20], EMolWt[20], EPCN[20], 
	  EPtot[20], EqRatio[20], ESnowMelt[20], ESnowPack[20], EStream[20], 
	  ESurfTot[20], ETFall[20], Etot[20], ETotOrg[20], EUp[20], 
	  EWeather[20], EWetDep[20], fEEx[20], fEFU[20], MineralMass, 
	  PassiveOM, ReactionStartYr, RootSinkEff[20], SolidPhase[20], SSFlux[21], //? SSFlux[0:20]
	  totdep[20], totLit[20], totMin[20], totTF[20], totUp[20], 
	  VolSoil, VolStream;
*/

double AvgSoil[100], AvgStream[41], AvSolid[100], CO2Prod, DHO,
      DOCPart, DOCProd, EDep[100], EDryDepRatio[100], EFastFlow[100],
	  EFCN[100], EFolEx[100], EFolUp[100], EHumus[100], 
	  Eleach[100], ELit[100], EMin[100], EMolWt[100], EPCN[100], 
	  EPtot[100], EqRatio[100], ESnowMelt[100], ESnowPack[100], EStream[100], 
	  ESurfTot[100], ETFall[100], Etot[100], ETotOrg[100], EUp[100], 
	  EWeather[100], EWetDep[100], fEEx[100], fEFU[100], MineralMass, 
	  PassiveOM,  RootSinkEff[100], SolidPhase[100], SSFlux[100], //? SSFlux[0:21]
	  totdep[100], totLit[100], totMin[100], totTF[100], totUp[100], 
	  VolSoil, VolStream;  // all the dimensions are changed from  21 to 100

int ReactionStartYr;

ifstream infile1, infile2, infile3, infile4, infile5, infile6, infile7, infile8, infile9;
//output files
ofstream outfile10, outfile11, outfile12, outfile13, outfileAdsorption;
ofstream outfile70; //Afshin Pourmokhtarian added 11/11/09
ofstream outfile80; //Afshin Pourmokhtarian added 11/11/09
ofstream outfile30, outfile40; //Afshin Pourmokhtarian added 06/23/10
